/*
	This file contains the neural network specific routines.
	Only neuralPlacement() remains in workspace.cpp for consistency,
	and the helper functions here for code cleanliness.
	
	Nicolas Brodu, brodu@kde.org, 19 dec 1999
*/

#include <kconfig.h>
#include <kglobal.h>

// Added for NeuralPlacement propagation and training
#include <math.h>
#include <time.h>
#include <fstream.h>

#include "workspace.h"


/*!
  Helper function to train the network
  When the network calculated incorrect values, call this
  with targetX and targetY as the desired values
  First backpropagation is done
  Then gradient descent with a very small learning rate
 */
void Workspace::trainNeuralNetwork(double targetX, double targetY)
{
	const double rate=0.001;
	QRect geom = geometry();
	double desktopWidth= geom.width();
	double desktopHeight = geom.height();
	
/*
 * Phase one: Backpropagation
 */
	
	// deltas
	double dhidden[neuralNetwork.hiddenDim];
	double doutput[neuralNetwork.outputDim];
	
	// linear output function => straightforward
	// However, normalize the targets (resolution independance)
	doutput[0]=neuralNetwork.output[0]-targetX/desktopWidth;
	doutput[1]=neuralNetwork.output[1]-targetY/desktopHeight;
	
	// backpropagation in hidden units
	for (int i=0; i<neuralNetwork.hiddenDim; i++) {
		double total=0;
		for (int j=0; j<neuralNetwork.outputDim; j++) {
			total+=doutput[j]*neuralNetwork.weightsLayer2[i][j];
		}
		// derivative of tanh is 1-tanh^2
		dhidden[i]=total*(1.0-neuralNetwork.hidden[i]*neuralNetwork.hidden[i]);
	}

	// Calculate the gradients in O(number of param)
	double ghiddenBias[neuralNetwork.hiddenDim];
	double goutputBias[neuralNetwork.outputDim];
	double gweightsLayer1[neuralNetwork.inputDim][neuralNetwork.hiddenDim];
	double gweightsLayer2[neuralNetwork.hiddenDim][neuralNetwork.outputDim];

	// first later
	for (int j=0; j<neuralNetwork.hiddenDim; j++) {
		for (int i=0; i<neuralNetwork.inputDim; i++) {
			gweightsLayer1[i][j]=dhidden[j]*neuralNetwork.input[i];
		}
		ghiddenBias[j]=dhidden[j];
	}
	// second later
	for (int j=0; j<neuralNetwork.outputDim; j++) {
		for (int i=0; i<neuralNetwork.hiddenDim; i++) {
			gweightsLayer2[i][j]=doutput[j]*neuralNetwork.hidden[i];
		}
		goutputBias[j]=doutput[j];
	}


/*
 * Phase two: gradient descent
 */
	// first later
	for (int j=0; j<neuralNetwork.hiddenDim; j++) {
		for (int i=0; i<neuralNetwork.inputDim; i++) {
			neuralNetwork.weightsLayer1[i][j]-=rate*gweightsLayer1[i][j];
		}
		neuralNetwork.hiddenBias[j]-=rate*ghiddenBias[j];
	}
	// second later
	for (int j=0; j<neuralNetwork.outputDim; j++) {
		for (int i=0; i<neuralNetwork.hiddenDim; i++) {
			neuralNetwork.weightsLayer2[i][j]-=rate*gweightsLayer2[i][j];
		}
		neuralNetwork.outputBias[j]-=rate*goutputBias[j];
	}
}


/*!
  Load the network from the "neuralplacementrc" config file
  Hmmm... KConfig doesn't seem to load this file properly (3 dec 99)
  => use KGlobal::config() instead
 */
void Workspace::loadNeuralNetwork()
{
    neuralNetwork.targetX=-geometry().width();
    neuralNetwork.targetY=-geometry().height();
	neuralNetwork.lastPlacementTime=0;
	neuralNetwork.lastPlacedClient=0;

    KConfig *config = KGlobal::config();
	config->setGroup("Neural Network");
	
	QString key;
	for (int i=0; i<neuralNetwork.hiddenDim; i++) {
		key.sprintf("hiddenBias[%d]",i);
		neuralNetwork.hiddenBias[i] = config->readDoubleNumEntry(key);
    }
	for (int i=0; i<neuralNetwork.outputDim; i++) {
		key.sprintf("outputBias[%d]",i);
		neuralNetwork.outputBias[i] = config->readDoubleNumEntry(key);
    }
	
	// The weights should not be 0, so set a flag
	bool corrupted = false;
	for (int i=0; i<neuralNetwork.inputDim; i++) {
		for (int j=0; j<neuralNetwork.hiddenDim; j++) {
			key.sprintf("weightsLayer1[%d][%d]",i,j);
			double entry = config->readDoubleNumEntry(key);
			neuralNetwork.weightsLayer1[i][j] = entry;
			if (entry==0) corrupted = true;
		}
    }
	for (int i=0; i<neuralNetwork.hiddenDim; i++) {
		for (int j=0; j<neuralNetwork.outputDim; j++) {
			key.sprintf("weightsLayer2[%d][%d]",i,j);
			double entry = config->readDoubleNumEntry(key);
			neuralNetwork.weightsLayer2[i][j] = entry;
			if (entry==0) corrupted = true;
		}
    }
	if (corrupted) resetNeuralNetwork();
}

/*!
  Save the network in a config file.
  Same comment as for the load function.
 */
void Workspace::saveNeuralNetwork()
{
    KConfig *config = KGlobal::config();
	config->setGroup("Neural Network");
	
	QString key;
	for (int i=0; i<neuralNetwork.hiddenDim; i++) {
		key.sprintf("hiddenBias[%d]",i);
		config->writeEntry(key, neuralNetwork.hiddenBias[i]);
    }
	for (int i=0; i<neuralNetwork.outputDim; i++) {
		key.sprintf("outputBias[%d]",i);
		config->writeEntry(key, neuralNetwork.outputBias[i]);
	}
	for (int i=0; i<neuralNetwork.inputDim; i++) {
		for (int j=0; j<neuralNetwork.hiddenDim; j++) {
			key.sprintf("weightsLayer1[%d][%d]",i,j);
		config->writeEntry(key, neuralNetwork.weightsLayer1[i][j]);
		}
    }
	for (int i=0; i<neuralNetwork.hiddenDim; i++) {
		for (int j=0; j<neuralNetwork.outputDim; j++) {
			key.sprintf("weightsLayer2[%d][%d]",i,j);
		config->writeEntry(key, neuralNetwork.weightsLayer2[i][j]);
		}
    }
}


/*!
  Reset the network to a default state where it tries to approximate the
  'smart' placement policy.
  This function is used when the config file doesn't exist/is corrupted.
 */
void Workspace::resetNeuralNetwork()
{
	neuralNetwork.hiddenBias[0]=-0.0571118;
	neuralNetwork.hiddenBias[1]=-0.176973;
	neuralNetwork.hiddenBias[2]=-0.0618831;
	neuralNetwork.hiddenBias[3]=0.344443;
	neuralNetwork.hiddenBias[4]=0.125231;
	neuralNetwork.hiddenBias[5]=-0.0769208;
	neuralNetwork.hiddenBias[6]=-0.279656;
	neuralNetwork.hiddenBias[7]=-0.23145;
	neuralNetwork.hiddenBias[8]=-0.159648;
	neuralNetwork.hiddenBias[9]=0.302131;
	neuralNetwork.outputBias[0]=0.0985691;
	neuralNetwork.outputBias[1]=-0.191102;
	neuralNetwork.weightsLayer1[0][0]=-0.00586404;
	neuralNetwork.weightsLayer1[0][1]=-0.0377446;
	neuralNetwork.weightsLayer1[0][2]=-0.00142151;
	neuralNetwork.weightsLayer1[0][3]=0.0188158;
	neuralNetwork.weightsLayer1[0][4]=0.0121849;
	neuralNetwork.weightsLayer1[0][5]=0.0211324;
	neuralNetwork.weightsLayer1[0][6]=-0.00466337;
	neuralNetwork.weightsLayer1[0][7]=-0.00333757;
	neuralNetwork.weightsLayer1[0][8]=0.0186566;
	neuralNetwork.weightsLayer1[0][9]=-0.00542747;
	neuralNetwork.weightsLayer1[1][0]=0.00613805;
	neuralNetwork.weightsLayer1[1][1]=-0.00800871;
	neuralNetwork.weightsLayer1[1][2]=0.0369308;
	neuralNetwork.weightsLayer1[1][3]=-0.0336576;
	neuralNetwork.weightsLayer1[1][4]=-0.0109446;
	neuralNetwork.weightsLayer1[1][5]=-0.0109961;
	neuralNetwork.weightsLayer1[1][6]=-0.00287906;
	neuralNetwork.weightsLayer1[1][7]=-0.00348111;
	neuralNetwork.weightsLayer1[1][8]=-0.00157423;
	neuralNetwork.weightsLayer1[1][9]=0.0111786;
	neuralNetwork.weightsLayer1[2][0]=-0.00661577;
	neuralNetwork.weightsLayer1[2][1]=0.00197781;
	neuralNetwork.weightsLayer1[2][2]=-0.00807602;
	neuralNetwork.weightsLayer1[2][3]=-0.00353308;
	neuralNetwork.weightsLayer1[2][4]=-0.0330198;
	neuralNetwork.weightsLayer1[2][5]=0.0385126;
	neuralNetwork.weightsLayer1[2][6]=-0.00388795;
	neuralNetwork.weightsLayer1[2][7]=0.00132832;
	neuralNetwork.weightsLayer1[2][8]=0.00669574;
	neuralNetwork.weightsLayer1[2][9]=-0.000448121;
	neuralNetwork.weightsLayer1[3][0]=-0.0122577;
	neuralNetwork.weightsLayer1[3][1]=-0.014654;
	neuralNetwork.weightsLayer1[3][2]=-1.13827e-05;
	neuralNetwork.weightsLayer1[3][3]=-0.000481632;
	neuralNetwork.weightsLayer1[3][4]=0.00757675;
	neuralNetwork.weightsLayer1[3][5]=0.0103098;
	neuralNetwork.weightsLayer1[3][6]=-0.00696507;
	neuralNetwork.weightsLayer1[3][7]=0.00148411;
	neuralNetwork.weightsLayer1[3][8]=0.0169072;
	neuralNetwork.weightsLayer1[3][9]=-0.000230242;
	neuralNetwork.weightsLayer1[4][0]=-0.0121424;
	neuralNetwork.weightsLayer1[4][1]=0.0410648;
	neuralNetwork.weightsLayer1[4][2]=0.0162004;
	neuralNetwork.weightsLayer1[4][3]=-0.00996311;
	neuralNetwork.weightsLayer1[4][4]=-0.00465743;
	neuralNetwork.weightsLayer1[4][5]=0.0378849;
	neuralNetwork.weightsLayer1[4][6]=-0.00658141;
	neuralNetwork.weightsLayer1[4][7]=-0.00432204;
	neuralNetwork.weightsLayer1[4][8]=0.00955186;
	neuralNetwork.weightsLayer1[4][9]=-0.0122008;
	neuralNetwork.weightsLayer1[5][0]=-0.0283883;
	neuralNetwork.weightsLayer1[5][1]=-0.0370504;
	neuralNetwork.weightsLayer1[5][2]=0.0180558;
	neuralNetwork.weightsLayer1[5][3]=-0.00146319;
	neuralNetwork.weightsLayer1[5][4]=-0.0277611;
	neuralNetwork.weightsLayer1[5][5]=0.0163476;
	neuralNetwork.weightsLayer1[5][6]=-0.00542015;
	neuralNetwork.weightsLayer1[5][7]=-0.0143448;
	neuralNetwork.weightsLayer1[5][8]=-0.0307175;
	neuralNetwork.weightsLayer1[5][9]=0.00446676;
	neuralNetwork.weightsLayer1[6][0]=0.00863532;
	neuralNetwork.weightsLayer1[6][1]=0.0020292;
	neuralNetwork.weightsLayer1[6][2]=-0.007889;
	neuralNetwork.weightsLayer1[6][3]=-0.0203568;
	neuralNetwork.weightsLayer1[6][4]=-0.0216855;
	neuralNetwork.weightsLayer1[6][5]=0.00434897;
	neuralNetwork.weightsLayer1[6][6]=0.00147528;
	neuralNetwork.weightsLayer1[6][7]=0.00042924;
	neuralNetwork.weightsLayer1[6][8]=-0.0105572;
	neuralNetwork.weightsLayer1[6][9]=-0.00468157;
	neuralNetwork.weightsLayer1[7][0]=0.00691135;
	neuralNetwork.weightsLayer1[7][1]=0.00493803;
	neuralNetwork.weightsLayer1[7][2]=0.0179793;
	neuralNetwork.weightsLayer1[7][3]=0.023744;
	neuralNetwork.weightsLayer1[7][4]=-0.00652067;
	neuralNetwork.weightsLayer1[7][5]=0.00347802;
	neuralNetwork.weightsLayer1[7][6]=-0.00280773;
	neuralNetwork.weightsLayer1[7][7]=0.00303467;
	neuralNetwork.weightsLayer1[7][8]=0.0049469;
	neuralNetwork.weightsLayer1[7][9]=0.000748864;
	neuralNetwork.weightsLayer1[8][0]=-0.00175176;
	neuralNetwork.weightsLayer1[8][1]=0.0229018;
	neuralNetwork.weightsLayer1[8][2]=0.00288959;
	neuralNetwork.weightsLayer1[8][3]=0.00430076;
	neuralNetwork.weightsLayer1[8][4]=-0.0080788;
	neuralNetwork.weightsLayer1[8][5]=0.0181843;
	neuralNetwork.weightsLayer1[8][6]=0.00241949;
	neuralNetwork.weightsLayer1[8][7]=-0.0103295;
	neuralNetwork.weightsLayer1[8][8]=0.00445745;
	neuralNetwork.weightsLayer1[8][9]=-0.00844243;
	neuralNetwork.weightsLayer1[9][0]=0.00201641;
	neuralNetwork.weightsLayer1[9][1]=-0.0135834;
	neuralNetwork.weightsLayer1[9][2]=0.000131477;
	neuralNetwork.weightsLayer1[9][3]=0.00857172;
	neuralNetwork.weightsLayer1[9][4]=0.0111449;
	neuralNetwork.weightsLayer1[9][5]=0.0109635;
	neuralNetwork.weightsLayer1[9][6]=-0.00034596;
	neuralNetwork.weightsLayer1[9][7]=0.0171336;
	neuralNetwork.weightsLayer1[9][8]=0.00612253;
	neuralNetwork.weightsLayer1[9][9]=-0.00722977;
	neuralNetwork.weightsLayer1[10][0]=-0.00475882;
	neuralNetwork.weightsLayer1[10][1]=0.00321184;
	neuralNetwork.weightsLayer1[10][2]=0.0131276;
	neuralNetwork.weightsLayer1[10][3]=0.000114081;
	neuralNetwork.weightsLayer1[10][4]=0.000756569;
	neuralNetwork.weightsLayer1[10][5]=0.0251498;
	neuralNetwork.weightsLayer1[10][6]=0.00075316;
	neuralNetwork.weightsLayer1[10][7]=-0.0140693;
	neuralNetwork.weightsLayer1[10][8]=0.0220163;
	neuralNetwork.weightsLayer1[10][9]=-0.00546873;
	neuralNetwork.weightsLayer1[11][0]=0.0023019;
	neuralNetwork.weightsLayer1[11][1]=0.0174016;
	neuralNetwork.weightsLayer1[11][2]=2.11788e-05;
	neuralNetwork.weightsLayer1[11][3]=0.011431;
	neuralNetwork.weightsLayer1[11][4]=0.0367031;
	neuralNetwork.weightsLayer1[11][5]=-0.00907718;
	neuralNetwork.weightsLayer1[11][6]=0.00121461;
	neuralNetwork.weightsLayer1[11][7]=-0.0161694;
	neuralNetwork.weightsLayer1[11][8]=0.00807917;
	neuralNetwork.weightsLayer1[11][9]=0.0130806;
	neuralNetwork.weightsLayer1[12][0]=0.00325119;
	neuralNetwork.weightsLayer1[12][1]=0.012578;
	neuralNetwork.weightsLayer1[12][2]=-0.0087877;
	neuralNetwork.weightsLayer1[12][3]=0.0126478;
	neuralNetwork.weightsLayer1[12][4]=0.00109186;
	neuralNetwork.weightsLayer1[12][5]=-0.00559944;
	neuralNetwork.weightsLayer1[12][6]=0.00468999;
	neuralNetwork.weightsLayer1[12][7]=-0.0195209;
	neuralNetwork.weightsLayer1[12][8]=-0.00404083;
	neuralNetwork.weightsLayer1[12][9]=-0.00452964;
	neuralNetwork.weightsLayer1[13][0]=0.00835907;
	neuralNetwork.weightsLayer1[13][1]=0.0213462;
	neuralNetwork.weightsLayer1[13][2]=-0.00739407;
	neuralNetwork.weightsLayer1[13][3]=0.0230619;
	neuralNetwork.weightsLayer1[13][4]=-0.0133615;
	neuralNetwork.weightsLayer1[13][5]=-0.00408724;
	neuralNetwork.weightsLayer1[13][6]=-0.00407918;
	neuralNetwork.weightsLayer1[13][7]=0.00723503;
	neuralNetwork.weightsLayer1[13][8]=-0.0028832;
	neuralNetwork.weightsLayer1[13][9]=-0.00441171;
	neuralNetwork.weightsLayer1[14][0]=0.0287445;
	neuralNetwork.weightsLayer1[14][1]=0.0137843;
	neuralNetwork.weightsLayer1[14][2]=0.006806;
	neuralNetwork.weightsLayer1[14][3]=-0.00468217;
	neuralNetwork.weightsLayer1[14][4]=-0.0479602;
	neuralNetwork.weightsLayer1[14][5]=0.0267656;
	neuralNetwork.weightsLayer1[14][6]=-0.00111866;
	neuralNetwork.weightsLayer1[14][7]=0.00547215;
	neuralNetwork.weightsLayer1[14][8]=-0.00619006;
	neuralNetwork.weightsLayer1[14][9]=-0.0230013;
	neuralNetwork.weightsLayer1[15][0]=0.00545269;
	neuralNetwork.weightsLayer1[15][1]=0.0166653;
	neuralNetwork.weightsLayer1[15][2]=0.0224487;
	neuralNetwork.weightsLayer1[15][3]=0.0731787;
	neuralNetwork.weightsLayer1[15][4]=0.0156815;
	neuralNetwork.weightsLayer1[15][5]=0.0219135;
	neuralNetwork.weightsLayer1[15][6]=0.0061268;
	neuralNetwork.weightsLayer1[15][7]=0.0108259;
	neuralNetwork.weightsLayer1[15][8]=0.0245602;
	neuralNetwork.weightsLayer1[15][9]=0.00230199;
	neuralNetwork.weightsLayer1[16][0]=-0.00507581;
	neuralNetwork.weightsLayer1[16][1]=0.00168189;
	neuralNetwork.weightsLayer1[16][2]=0.00077881;
	neuralNetwork.weightsLayer1[16][3]=-0.00103551;
	neuralNetwork.weightsLayer1[16][4]=0.0101437;
	neuralNetwork.weightsLayer1[16][5]=0.0217152;
	neuralNetwork.weightsLayer1[16][6]=0.00570211;
	neuralNetwork.weightsLayer1[16][7]=-0.0161667;
	neuralNetwork.weightsLayer1[16][8]=0.000661458;
	neuralNetwork.weightsLayer1[16][9]=0.00062554;
	neuralNetwork.weightsLayer1[17][0]=-7.70504e-05;
	neuralNetwork.weightsLayer1[17][1]=-0.0100909;
	neuralNetwork.weightsLayer1[17][2]=0.00554086;
	neuralNetwork.weightsLayer1[17][3]=-0.0148261;
	neuralNetwork.weightsLayer1[17][4]=-0.00197658;
	neuralNetwork.weightsLayer1[17][5]=0.0330684;
	neuralNetwork.weightsLayer1[17][6]=0.00114943;
	neuralNetwork.weightsLayer1[17][7]=-0.00677443;
	neuralNetwork.weightsLayer1[17][8]=-0.00660057;
	neuralNetwork.weightsLayer1[17][9]=-0.00869773;
	neuralNetwork.weightsLayer1[18][0]=-0.000498323;
	neuralNetwork.weightsLayer1[18][1]=0.0021369;
	neuralNetwork.weightsLayer1[18][2]=0.0171355;
	neuralNetwork.weightsLayer1[18][3]=-0.000105287;
	neuralNetwork.weightsLayer1[18][4]=-0.00838526;
	neuralNetwork.weightsLayer1[18][5]=-0.00431244;
	neuralNetwork.weightsLayer1[18][6]=-0.0042915;
	neuralNetwork.weightsLayer1[18][7]=-0.0127976;
	neuralNetwork.weightsLayer1[18][8]=-0.0100739;
	neuralNetwork.weightsLayer1[18][9]=0.00502674;
	neuralNetwork.weightsLayer1[19][0]=0.0245335;
	neuralNetwork.weightsLayer1[19][1]=0.0266611;
	neuralNetwork.weightsLayer1[19][2]=0.00808358;
	neuralNetwork.weightsLayer1[19][3]=0.0200505;
	neuralNetwork.weightsLayer1[19][4]=-0.0557838;
	neuralNetwork.weightsLayer1[19][5]=0.0124623;
	neuralNetwork.weightsLayer1[19][6]=-0.00793221;
	neuralNetwork.weightsLayer1[19][7]=0.000317576;
	neuralNetwork.weightsLayer1[19][8]=-0.0194577;
	neuralNetwork.weightsLayer1[19][9]=-0.00264442;
	neuralNetwork.weightsLayer1[20][0]=-0.0230376;
	neuralNetwork.weightsLayer1[20][1]=-0.0102106;
	neuralNetwork.weightsLayer1[20][2]=0.0203301;
	neuralNetwork.weightsLayer1[20][3]=0.0208163;
	neuralNetwork.weightsLayer1[20][4]=-0.0209696;
	neuralNetwork.weightsLayer1[20][5]=0.0234038;
	neuralNetwork.weightsLayer1[20][6]=0.000492031;
	neuralNetwork.weightsLayer1[20][7]=-0.0184014;
	neuralNetwork.weightsLayer1[20][8]=0.0140066;
	neuralNetwork.weightsLayer1[20][9]=-0.008409;
	neuralNetwork.weightsLayer1[21][0]=-0.0522779;
	neuralNetwork.weightsLayer1[21][1]=-0.0150072;
	neuralNetwork.weightsLayer1[21][2]=0.00670097;
	neuralNetwork.weightsLayer1[21][3]=0.0246407;
	neuralNetwork.weightsLayer1[21][4]=0.00355295;
	neuralNetwork.weightsLayer1[21][5]=-0.011569;
	neuralNetwork.weightsLayer1[21][6]=-0.00612842;
	neuralNetwork.weightsLayer1[21][7]=0.0480664;
	neuralNetwork.weightsLayer1[21][8]=-0.0474313;
	neuralNetwork.weightsLayer1[21][9]=0.0236559;
	neuralNetwork.weightsLayer1[22][0]=0.00515591;
	neuralNetwork.weightsLayer1[22][1]=0.0223685;
	neuralNetwork.weightsLayer1[22][2]=0.00489221;
	neuralNetwork.weightsLayer1[22][3]=-0.00246388;
	neuralNetwork.weightsLayer1[22][4]=-0.0182339;
	neuralNetwork.weightsLayer1[22][5]=-0.0221762;
	neuralNetwork.weightsLayer1[22][6]=0.0215816;
	neuralNetwork.weightsLayer1[22][7]=0.0314824;
	neuralNetwork.weightsLayer1[22][8]=0.00369286;
	neuralNetwork.weightsLayer1[22][9]=0.0292745;
	neuralNetwork.weightsLayer2[0][0]=0.148176;
	neuralNetwork.weightsLayer2[0][1]=-0.0255225;
	neuralNetwork.weightsLayer2[1][0]=0.114791;
	neuralNetwork.weightsLayer2[1][1]=-0.036562;
	neuralNetwork.weightsLayer2[2][0]=0.0762642;
	neuralNetwork.weightsLayer2[2][1]=0.0439302;
	neuralNetwork.weightsLayer2[3][0]=0.0218994;
	neuralNetwork.weightsLayer2[3][1]=0.239032;
	neuralNetwork.weightsLayer2[4][0]=-0.152386;
	neuralNetwork.weightsLayer2[4][1]=-0.0249094;
	neuralNetwork.weightsLayer2[5][0]=0.00735878;
	neuralNetwork.weightsLayer2[5][1]=0.261785;
	neuralNetwork.weightsLayer2[6][0]=0.0353309;
	neuralNetwork.weightsLayer2[6][1]=-0.165639;
	neuralNetwork.weightsLayer2[7][0]=-0.0468058;
	neuralNetwork.weightsLayer2[7][1]=-0.0831551;
	neuralNetwork.weightsLayer2[8][0]=0.0863193;
	neuralNetwork.weightsLayer2[8][1]=-0.0258391;
	neuralNetwork.weightsLayer2[9][0]=-0.0842569;
	neuralNetwork.weightsLayer2[9][1]=-0.0506152;
}
